Public Delegate Function MyFxDelegate(ByVal nNumVars As Integer, ByRef fX() As Double, ByRef fParam() As Double) As Double
Public Delegate Function SayFxDelegate() As String

Public Class CHookJeevesSearch1
  Dim m_MyFx As MyFxDelegate

  Protected Function MyFxEx(ByVal nNumVars As Integer, _
                ByRef fX() As Double, ByRef fParam() As Double, _
                ByRef fDeltaX() As Double, ByVal fLambda As Double) As Double
    Dim I As Integer

    Dim fXX(nNumVars) As Double

    For I = 0 To nNumVars - 1
      fXX(I) = fX(I) + fLambda * fDeltaX(I)
    Next I

    MyFxEx = m_MyFx(nNumVars, fXX, fParam)
  End Function

  Protected Function LinSearch_DirectSearch(ByVal nNumVars As Integer, ByRef fX() As Double, ByRef fParam() As Double, _
          ByRef fLambda As Double, ByRef fDeltaX() As Double, ByVal fInitStep As Double, ByVal fMinStep As Double) As Boolean
    Dim F1, F2 As Double

    F1 = MyFxEx(nNumVars, fX, fParam, fDeltaX, fLambda)

    Do
      F2 = MyFxEx(nNumVars, fX, fParam, fDeltaX, fLambda + fInitStep)
      If F2 < F1 Then
        F1 = F2
        fLambda += fInitStep
      Else
        F2 = MyFxEx(nNumVars, fX, fParam, fDeltaX, fLambda - fInitStep)
        If F2 < F1 Then
          F1 = F2
          fLambda -= fInitStep
        Else
          ' reduce search step size
          fInitStep /= 10
        End If
      End If
    Loop Until fInitStep < fMinStep

    Return True

  End Function

  Public Function CalcOptim(ByVal nNumVars As Integer, ByRef fX() As Double, ByRef fParam() As Double, _
                  ByRef fStepSize() As Double, ByRef fMinStepSize() As Double, ByVal fEpsFx As Double, _
                  ByRef nIter As Integer, ByVal MyFx As MyFxDelegate) As Double
    Dim I As Integer
    Dim fXnew(nNumVars) As Double
    Dim fDeltaX(nNumVars) As Double
    Dim F As Double, fXX As Double, fLambda As Double
    Dim fBestF, fLastBestF As Double
    Dim bStop, bMadeAnyMove As Boolean, bMoved(nNumVars) As Boolean

    m_MyFx = MyFx

    For I = 0 To nNumVars - 1
      fXnew(I) = fX(I)
    Next
    ' calculate function value at initial point
    fBestF = MyFx(nNumVars, fXnew, fParam)
    fLastBestF = 100 * fBestF + 100

    nIter = 1
    Do

      nIter += 1

      For I = 0 To nNumVars - 1
        fX(I) = fXnew(I)
      Next I

      For I = 0 To nNumVars - 1
        bMoved(I) = False
        Do
          fXX = fXnew(I)
          fXnew(I) = fXX + fStepSize(I)
          F = MyFx(nNumVars, fXnew, fParam)
          If F < fBestF Then
            fBestF = F
            bMoved(I) = True
          Else
            fXnew(I) = fXX - fStepSize(I)
            F = MyFx(nNumVars, fXnew, fParam)
            If F < fBestF Then
              fBestF = F
              bMoved(I) = True
            Else
              fXnew(I) = fXX
              Exit Do
            End If
          End If
        Loop
      Next I

      ' moved in any direction?
      bMadeAnyMove = True
      For I = 0 To nNumVars - 1
        If Not bMoved(I) Then
          bMadeAnyMove = False
          Exit For
        End If
      Next I

      If bMadeAnyMove Then
        For I = 0 To nNumVars - 1
          fDeltaX(I) = fXnew(I) - fX(I)
        Next I

        fLambda = 0
        If LinSearch_DirectSearch(nNumVars, fX, fParam, fLambda, fDeltaX, 0.1, 0.0001) Then
          For I = 0 To nNumVars - 1
            fXnew(I) = fX(I) + fLambda * fDeltaX(I)
          Next I
        End If
      End If
      fBestF = MyFx(nNumVars, fXnew, fParam)

      ' reduce the step size for the dimensions that had no moves
      For I = 0 To nNumVars - 1
        If Not bMoved(I) Then fStepSize(I) /= 2
      Next I

      ' test function value convergence
      If Math.Abs(fBestF - fLastBestF) < fEpsFx Then Exit Do

      fLastBestF = fBestF

      bStop = True
      For I = 0 To nNumVars - 1
        If fStepSize(I) >= fMinStepSize(I) Then
          bStop = False
          Exit For
        End If
      Next I

    Loop Until bStop

    For I = 0 To nNumVars - 1
      fX(I) = fXnew(I)
    Next I

    Return fBestF

  End Function
End Class
